package cn.org.bjca.trust.android.lib.im.manager;

import android.content.Context;
import android.util.Log;

import cn.org.bjca.trust.android.lib.im.BuildConfig;
import cn.org.bjca.trust.android.lib.im.SZYXDbHelper;
import cn.org.bjca.trust.android.lib.im.cfg.Constant;
import cn.org.bjca.trust.android.lib.im.common.DeviceHelper;
import cn.org.bjca.trust.android.lib.im.http.HttpManage;
import cn.org.bjca.trust.android.lib.im.kit.IMSDKCallback;
import cn.org.bjca.trust.android.lib.im.kit.IMSDKListener;
import cn.org.bjca.trust.android.lib.im.kit.MsgListener;
import cn.org.bjca.trust.android.lib.im.kit.SdkInterface;
import cn.org.bjca.trust.android.lib.im.repository.Service;
import cn.org.bjca.trust.android.lib.im.repository.data.LoginData;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;
import io.reactivex.schedulers.Schedulers;

public class SZYXImManager implements SdkInterface {

    private IMSDKListener imsdkListener;

    @Override
    public void addIMSDKListener(IMSDKListener listener) {
        this.imsdkListener = listener;
    }

    @Override
    public void removeIMSDKListener(IMSDKListener listener) {
        this.imsdkListener = null;
    }

    @Override
    public void init(Context context, String sdkAppID, IMSDKCallback callback) {
        Constant.setSdkAppID(sdkAppID);
        SZYXDbHelper.get(context);
        DeviceHelper.getDevice();
    }

    @Override
    public void login(String userID, String userSig, IMSDKCallback callback) {
        Constant.setUserId(userID);
        Disposable d = HttpManage.getApi(Service.class)
                .login(new LoginData(userID, userSig, DeviceHelper.getDevice()))
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(httpResult -> {
                    if (httpResult.getCode() == 200) {
                        if (null != callback) callback.success();
                        if (null != imsdkListener) imsdkListener.onConnecting();
                    } else if (null != callback) callback.failed(1001, httpResult.getMsg());
                }, throwable -> {
                    if (null != callback) callback.failed(1001, throwable.getMessage());
                });
    }

    @Override
    public void logout(IMSDKCallback callback) {

    }

    @Override
    public void addMsgListener(MsgListener listener) {

    }

    @Override
    public void removeMsgListener(MsgListener listener) {

    }

    @Override
    public void sendMsgForTextToC(String toUserId, String text, IMSDKCallback callback) {

    }

    @Override
    public void sendMsgForTextToG(String toGroupId, String text, IMSDKCallback callback) {

    }

    @Override
    public String getVersion() {
        return BuildConfig.versionName;
    }
}
